package usecase

import (
	"math"
	"time"

	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type BimbinganLogUsecase interface {
	AddBimbinganLog(bimbinganLog entity.BimbinganLogBerkas) (entity.BimbinganLogBerkas, error)
	GetBimbinganLog(param utils.LimitOffset, idMahasiswa string) ([]entity.BimbinganLogBerkas, error)
	UpdateBimbinganLogStatus(id string, status bool) error
	GetInfoTopikMahasiswa(idMahasiswa string) (entity.InfoTopikMahasiswa, error)
	GetBimbinganLogStatusByStudentId(param utils.LimitOffset, idMahasiswa string) (entity.BimbinganLogStatus, error)
}

type BimbinganLogUc struct {
	bimbinganLogRepo BimbinganLogRepository
}

func NewBimbinganLogUseCase(bimbinganLogRepo BimbinganLogRepository) *BimbinganLogUc {
	return &BimbinganLogUc{
		bimbinganLogRepo: bimbinganLogRepo,
	}
}

func (uc *BimbinganLogUc) AddBimbinganLog(bimbinganLogBerkas entity.BimbinganLogBerkas) (entity.BimbinganLogBerkas, error) {
	berkas := bimbinganLogBerkas.Berkas
	bimbinganLog := entity.BimbinganLog{
		IDMahasiswa:     bimbinganLogBerkas.IDMahasiswa,
		Date:            bimbinganLogBerkas.Date,
		LaporanKemajuan: bimbinganLogBerkas.LaporanKemajuan,
		Todo:            bimbinganLogBerkas.Todo,
		NextBimbingan:   bimbinganLogBerkas.NextBimbingan,
	}

	newBimbinganLogBerkas, err := uc.bimbinganLogRepo.NewBimbinganLog(berkas, bimbinganLog)

	if err != nil {
		return newBimbinganLogBerkas, err
	}

	return newBimbinganLogBerkas, nil

}

func (uc *BimbinganLogUc) GetBimbinganLog(param utils.LimitOffset, nim string) ([]entity.BimbinganLogBerkas, error) {
	return uc.bimbinganLogRepo.GetBimbinganLogByStudentId(param.Limit, param.Offset, nim)
}

func (uc *BimbinganLogUc) UpdateBimbinganLogStatus(id string, status bool) error {
	return uc.bimbinganLogRepo.UpdateBimbinganLogStatus(id, status)
}

func (uc *BimbinganLogUc) GetInfoTopikMahasiswa(idMahasiswa string) (entity.InfoTopikMahasiswa, error) {
	return uc.bimbinganLogRepo.GetInfoTopikMahasiswa(idMahasiswa)
}

func (uc *BimbinganLogUc) GetBimbinganLogStatusByStudentId(param utils.LimitOffset, nim string) (entity.BimbinganLogStatus, error) {

	logs := []entity.BimbinganLogBerkas{}
	var offset int
	const limit = 100

	for {
		logBimbinganStatus, err := uc.bimbinganLogRepo.GetBimbinganLogByStudentId(limit, offset, nim)

		if err != nil {
			// Handle error
			break
		}

		logs = append(logs, logBimbinganStatus...)
		if len(logs) < limit {
			break
		}

		offset += limit
	}

	bimbinganLogStatus := entity.BimbinganLogStatus{
		Status:       true,
		LogBimbingan: logs,
	}

	if len(logs) == 0 {
		bimbinganLogStatus.Status = false
		return bimbinganLogStatus, nil
	}

	newestLog := logs[0]

	now := time.Now()
	newestLogDate, err := time.Parse(time.RFC3339, newestLog.Date)

	discrepancy := int(math.Abs(float64(now.Month() - newestLogDate.Month())))

	if discrepancy <= 1 {
		bimbinganLogStatus.Status = true
	} else if discrepancy > 1 && discrepancy < 3 {
		bimbinganLogStatus.Status = false
	} else if discrepancy >= 3 {
		bimbinganLogStatus.Status = false
	}

	if err != nil {
		return bimbinganLogStatus, err
	}

	return bimbinganLogStatus, nil
}
