package transport

import (
	"net/http"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/bimbingan_log/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type MahasiswaBimbinganLogHandler struct {
	bimbinganLogUsecase usecase.BimbinganLogUsecase
}

func NewMahasiswaBimginganLogHandler(cfg config.BimbinganLogTransportConfig) *MahasiswaBimbinganLogHandler {
	bimbinganLogRepository := repository.NewBimbinganLogRepository(cfg.DBWrite, cfg.DBRead)
	bimbinganLogUsecase := usecase.NewBimbinganLogUseCase(bimbinganLogRepository)

	return &MahasiswaBimbinganLogHandler{
		bimbinganLogUsecase: bimbinganLogUsecase,
	}

}

func (t *MahasiswaBimbinganLogHandler) MountMahasiswa(group *echo.Group) {
	group.POST("/add-bimbingan-log", t.AddBimbinganLog)
}

// AddBimbinganLog godoc
// @Summary Add a new bimbingan log
// @Description Add a new bimbingan log
// @Tags BimbinganLog
// @Accept json
// @Produce json
// @Param bimbinganLog body entity.BimbinganLog true "Bimbingan Log object"
// @Success 200 {object} entity.BimbinganLog "Bimbingan Log successfully added"
// @Router /bimbingan-log [post]
func (t *MahasiswaBimbinganLogHandler) AddBimbinganLog(c echo.Context) error {
	bimbinganLogBerkas := new(entity.BimbinganLogBerkas)

	if err := c.Bind(bimbinganLogBerkas); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newBimbinganLogBerkas, err := t.bimbinganLogUsecase.AddBimbinganLog(*bimbinganLogBerkas)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Bimbingan log added successfully", newBimbinganLogBerkas))
}
