package entity

import "time"

type Pendaftaran struct {
	Id          string `gorm:"type:uuid;primaryKey;default:gen_random_uuid();unique"` // UUID as a string
	IdTopik     string
	IdMahasiswa string
	IdDosen     string
	Status      *bool      `gorm:"default:null"`
	InterviewAt *time.Time `gorm:"default:null"`
	DecidedAt   *time.Time `gorm:"default:null"`
	CreatedAt   time.Time  `gorm:"autoCreateTime"` // Use this if you want to override the gorm.Model's CreatedAt
	UpdatedAt   time.Time  `gorm:"autoUpdateTime"` // Use this if you want to override the gorm.Model's UpdatedAt
	IdPeriode   string
}

type Topik struct {
	ID        string `json:"id"`
	Judul     string `json:"judul"`
	Deskripsi string `json:"deskripsi"`
	PengajuID string `json:"pengaju_id"`

	Pengguna Pengguna `json:"pengaju" gorm:"foreignKey:PengajuID"`
}

func (Topik) TableName() string {
	return "topik"
}

type PendaftaranGetResponse struct {
	Id              string
	Status          string
	InterviewAt     time.Time
	DecidedAt       time.Time
	WaktuPengiriman time.Time
	Judul           string
	Deskripsi       string
	IdDosen         string
	NamaDosen       string
	EmailDosen      string
}

type ParamValues struct {
	Limit  int
	Offset int
}

type PendaftaranAddRequest struct {
	IdTopik        string
	IdMahasiswa    string
	IdDosen        string
	JudulTopik     string
	DeskripsiTopik string
	JalurPilihan   string
}

type PostTopikResponse struct {
	Success bool   `json:"success"`
	Code    int    `json:"code"`
	Message string `json:"message"`
	Data    []struct {
		Id        string `json:"id"`
		Judul     string `json:"judul"`
		Deskripsi string `json:"deskripsi"`
		PengajuId string `json:"pengaju_id"`
		Pengaju   struct {
			Id    string `json:"id"`
			Nama  string `json:"nama"`
			Email string `json:"email"`
			Roles string `json:"roles"`
		} `json:"pengaju"`
	} `json:"data"`
}

type UpdatePendaftaranRequest struct {
	Id     string
	Status string
}

type WaktuInterviewUpdateRequest struct {
	Id          string
	InterviewAt time.Time
}

type RekapPendaftaran struct {
	ID          string    `json:"pendaftaran_id"`
	IDMahasiswa string    `json:"mahasiswa_id"`
	Nama        string    `json:"mahasiswa_nama"`
	Nim         string    `json:"nim"`
	Status      string    `json:"status"`
	InterviewAt time.Time `json:"jadwal_interview"`
}

type RekapPendaftaranRes struct {
	ID          string `json:"pendaftaran_id"`
	IDMahasiswa string `json:"mahasiswa_id"`
	Nama        string `json:"mahasiswa_nama"`
	Nim         string `json:"nim"`
	Status      string `json:"status"`
	InterviewAt string `json:"jadwal_interview"`
}

type RekapPendaftaranTimta struct {
	ID             string `json:"mahasiswa_id"`
	Nama           string `json:"mahasiswa_nama"`
	Nim            string `json:"nim"`
	PembimbingNama string `json:"pembimbing_nama"`
	Status         string `json:"status"`
	PendaftaranId  string `json:"pendaftaran_id"`
}

type RiwayatPendaftaran struct {
	ID              string `json:"id"`
	InterviewAt     string `json:"jadwalInterview"`
	JalurPilihan    string `json:"jalurPilihan"`
	WaktuPengiriman string `json:"waktuPengiriman"`
	Status          string `json:"status"`
	Judul           string `json:"judulTopik"`
	Deskripsi       string `json:"deskripsiTopik"`
}

type DosenPembimbing struct {
	IDPendaftaran string `json:"pendaftaran_id"`
	ID            string `json:"id"`
	Nama          string `json:"nama"`
}

type DosenPembimbings struct {
	ID    string `json:"id"`
	Nama  string `json:"nama"`
	Email string `json:"email"`
}
type RiwayatPendaftaranRes struct {
	ID              string            `json:"id"`
	InterviewAt     string            `json:"jadwalInterview"`
	JalurPilihan    string            `json:"jalurPilihan"`
	WaktuPengiriman string            `json:"waktuPengiriman"`
	Status          string            `json:"status"`
	Judul           string            `json:"judulTopik"`
	Deskripsi       string            `json:"deskripsiTopik"`
	DosenPembimbing []DosenPembimbing `json:"dosenPembimbing"`
}

type DetailRekapPendaftara struct {
	Judul           string `json:"judulTopik"`
	Deskripsi       string `json:"deskripsiTopik"`
	InterviewAt     string `json:"jadwalInterview"`
	Status          string `json:"status"`
	WaktuPengiriman string `json:"waktuPengiriman"`
	JalurPilihan    string `json:"jalurPilihan"`
}

type UpdateInterviewReq struct {
	IDMahasiswa string `json:"id_mahasiswa"`
	InterviewAt string `json:"interview_at"`
}

type UpdateStatusReq struct {
	IDMahasiswa string `json:"id_mahasiswa"`
	Status      string `json:"status"`
}

type UpdateDosbingReq struct {
	IdDosen       []string `json:"pembimbing_ids"`
	IDPendaftaran string   `json:"id_pendaftaran"`
}

type UpdateStatusByIdReq struct {
	IDPendaftaran string `json:"id_pendaftaran"`
	Status        string `json:"status"`
}

type Stats struct {
	Diterima     StatsItem `json:"diterima"`
	SedangProses StatsItem `json:"sedang_proses"`
	Ditolak      StatsItem `json:"ditolak"`
}

type StatsItem struct {
	Amount     int64 `json:"amount"`
	Percentage int64 `json:"percentage"`
}

type Pengguna struct {
	ID    string `gorm:"type:uuid;primaryKey" json:"id"`
	Nama  string `json:"nama"`
	Email string `json:"email"`
}

type DosenBimbingan struct {
	ID            string
	PendaftaranID string
	DosenID       string
}
