package transport

import (
	"net/http"
	"strconv"

	"github.com/labstack/echo/v4"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran_sidsem/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminPendaftaranSidSemHandler struct {
	pendaftaransidsemUsecase usecase.PendaftaranSidSemUsecase
}

func NewAdminJadwalSeminarHandler(cfg config.PendaftaranSidSemTransportConfig) *AdminPendaftaranSidSemHandler {
	pendaftaransidsemRepository := repository.NewPendaftaranSidSemRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaransidsemUsecase := usecase.NewPendaftaranSidSemUseCase(pendaftaransidsemRepository)

	return &AdminPendaftaranSidSemHandler{
		pendaftaransidsemUsecase: pendaftaransidsemUsecase,
	}

}

func (t *AdminPendaftaranSidSemHandler) MountAdmin(group *echo.Group) {
	group.GET("/pendaftaran-sidsem-periode", t.GetPendaftaranByPeriode)
	group.DELETE("/pendaftaran-sidsem", t.DeletePendaftaran)
	group.GET("/pendaftaran-sidang", t.GetPendaftaranSidangById)
	group.GET("/pendaftaran-seminar", t.GetPendaftaranSeminarById)
	group.PATCH("/pendaftaran-ruangan", t.EditRuangan)
	group.PATCH("/penguji-seminar", t.EditDosenPengujiSeminar)
	group.PATCH("/penguji-sidang", t.EditDosenPengujiSidang)
	group.PATCH("/pendaftaran-waktu", t.EditWaktu)
	group.PATCH("/pendaftaran-lulus", t.EditLulus)
}

func (t *AdminPendaftaranSidSemHandler) GetPendaftaranByPeriode(c echo.Context) error {
	idPeriode := c.QueryParam("id_periode")
	sizeStr := c.QueryParam("limit")
	offsetStr := c.QueryParam("offset")

	size, err := strconv.Atoi(sizeStr)
	if err != nil {
		size = 10
	}

	offset, err := strconv.Atoi(offsetStr)
	if err != nil {
		offset = 0
	}
	jadwal, err := t.pendaftaransidsemUsecase.GetPendaftaranByPeriode(idPeriode, size, offset)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) GetPendaftaranSidangById(c echo.Context) error {
	idSidang := c.QueryParam("id")

	jadwal, err := t.pendaftaransidsemUsecase.GetPendaftaranSidangById(idSidang)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) GetPendaftaranSeminarById(c echo.Context) error {
	idSeminar := c.QueryParam("id")

	jadwal, err := t.pendaftaransidsemUsecase.GetPendaftaranSeminarById(idSeminar)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) DeletePendaftaran(c echo.Context) error {
	request := new(entity.PendaftaranSidSem)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.DeletePendaftaran(request.ID)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully deleted", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) EditRuangan(c echo.Context) error {
	request := new(entity.EditRuanganRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditRuangan(request.Id, request.IdRuangan)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) EditDosenPengujiSeminar(c echo.Context) error {
	request := new(entity.EditDosenPengujiSeminarRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditDosenPengujiSeminar(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) EditDosenPengujiSidang(c echo.Context) error {
	request := new(entity.EditDosenPengujiSidangRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditDosenPengujiSidang(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) EditWaktu(c echo.Context) error {
	request := new(entity.EditWaktuRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditWaktu(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}

func (t *AdminPendaftaranSidSemHandler) EditLulus(c echo.Context) error {
	request := new(entity.EditLulusRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}
	jadwal, err := t.pendaftaransidsemUsecase.EditLulus(*request)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully edited", jadwal))
}
