package repository

import (
	"github.com/google/uuid"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/ruangan/entity"
	"gorm.io/gorm"
)

type RuanganRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewRuanganRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *RuanganRepo {
	return &RuanganRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *RuanganRepo) NewRuangan(ruangan entity.Ruangan) (output entity.Ruangan, err error) {
	ruangan.ID = uuid.New().String()

	result := repo.DBWrite.Create(&ruangan)
	if result.Error != nil {
		return entity.Ruangan{}, result.Error
	}
	return ruangan, nil
}

func (repo *RuanganRepo) GetRuangan(limit int, offset int) (output []entity.Ruangan, err error) {
	var ruangan []entity.Ruangan
	result := repo.DBRead.Limit(limit).Offset(offset).Find(&ruangan)
	if result.Error != nil {
		return nil, err
	}
	return ruangan, nil
}

func (repo *RuanganRepo) GetRuanganById(id string) (output entity.Ruangan, err error) {
	var ruangan entity.Ruangan
	result := repo.DBRead.Where("id = ?", id).Find(&ruangan)
	if result.Error != nil {
		return entity.Ruangan{}, err
	}
	return ruangan, nil
}

func (repo *RuanganRepo) GetAllRuangan() (output []entity.Ruangan, err error) {
	var ruangan []entity.Ruangan
	result := repo.DBRead.Find(&ruangan)
	if result.Error != nil {
		return nil, err
	}
	return ruangan, nil
}
