package repository

import (
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/scheduling/entity"
	"gorm.io/gorm"
)

type SchedulingRepo struct {
	DBWrite *gorm.DB
	DBRead  *gorm.DB
}

func NewSchedulingRepository(dbWrite *gorm.DB, dbRead *gorm.DB) *SchedulingRepo {
	return &SchedulingRepo{
		DBWrite: dbWrite,
		DBRead:  dbRead,
	}
}

func (repo *SchedulingRepo) NewSchedulingLinkSidang(idAndLinkSidang entity.SchedulingSidangLink) error {
	result := repo.DBWrite.Create(&idAndLinkSidang)
	if result.Error != nil {
		return result.Error
	}

	return nil
}

func (repo *SchedulingRepo) NewSchedulingLinkSeminar(idAndLinkSeminar entity.SchedulingSeminarLink) error {
	result := repo.DBWrite.Create(&idAndLinkSeminar)
	if result.Error != nil {
		return result.Error
	}

	return nil
}

func (repo *SchedulingRepo) GetSchedulingTableData() (sidangResult []entity.JadwalWithLink, seminarResult []entity.JadwalWithLink, err error) {

	querySidang := "SELECT js.id, js.id_mahasiswa, js.id_dosbing, js.id_dosuji1, js.id_dosuji2, js.ruang, js.id_batch, js.judul, ss.link FROM jadwal_sidangs js JOIN scheduling_sidang ss ON js.id = ss.id;"

	querySeminar := "SELECT js.id, js.id_mahasiswa, js.id_dosbing, js.id_dosuji, js.ruang, js.id_batch, js.judul, ss.link FROM jadwal_seminars js JOIN scheduling_seminar ss ON js.id = ss.id;"

	if err := repo.DBRead.Raw(querySidang).Scan(&sidangResult).Error; err != nil {
		return []entity.JadwalWithLink{}, []entity.JadwalWithLink{}, err
	}

	if err := repo.DBRead.Raw(querySeminar).Scan(&seminarResult).Error; err != nil {
		return []entity.JadwalWithLink{}, []entity.JadwalWithLink{}, err
	}

	return sidangResult, seminarResult, err
}
